# Generated by Django 3.1.4 on 2021-06-04 05:58

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ("user", "0007_userinvite_to"),
    ]

    operations = [
        migrations.AlterField(
            model_name="userinvite",
            name="key",
            field=models.UUIDField(default=uuid.uuid4, unique=True),
        ),
    ]
