export const BIBLIOGRAPHY_HEADERS = {
    'af-ZA': 'Bibliografie',
    'sq-AL': 'Bibliografi',
    'ar': 'قائمة المراجع',
    'ast': 'Bibliografía',
    'be': 'Бібліяграфія',
    'br': 'Levrlennadur',
    'bg': 'библиография',
    'ca': 'Bibliografia',
    'ca-ES-Valencia': 'Bibliografia',
    'zh-CN': '参考书目',
    'da': 'Bibliografi',
    'nl': 'Bibliografie',
    'en-AU': 'Bibliography',
    'en-CA': 'Bibliography',
    'en-NZ': 'Bibliography',
    'en-ZA': 'Bibliography',
    'en-GB': 'Bibliography',
    'en-US': 'Bibliography',
    'eo': 'Bibliografio',
    'fr': 'Bibliographie',
    'gl': 'Bibliografía',
    'de-DE': 'Literaturverzeichnis',
    'de-AU': 'Literaturverzeichnis',
    'de-CH': 'Literaturverzeichnis',
    'el': 'Βιβλιογραφία',
    'he': 'ביבליוגרפיה',
    'is': 'Bókaskrá',
    'it': 'Bibliografia',
    'ja': '書誌',
    'km': 'គន្ថនិទ្ទេស',
    'lt': 'Bibliografija',
    'ml': 'ഗ്രന്ഥസൂചിക',
    'nb-NO': 'Bibliografi',
    'nn-NO': 'Bibliografi',
    'fa': 'کتابشناسی',
    'pl': 'Bibliografia',
    'pt-BR': 'Bibliografia',
    'pt-PT': 'Bibliografia',
    'ro': 'Bibliografie',
    'ru': 'библиография',
    'tr': 'Kaynakça',
    'sr-SP-Cy': 'Библиографија',
    'sr-SP-Lt': 'Bibliografija',
    'sk': 'Bibliografia',
    'sl': 'Bibliografija',
    'es': 'Bibliografía',
    'sv': 'Bibliografi',
    'ta': 'நூற்பட்டியல்',
    'tl': 'Bibliograpiya',
    'uk': 'Бібліографія'
}

const FIGURE = {
    'af-ZA': 'Figuur',
    'sq-AL': 'figurë',
    'ar': 'الشكل',
    'ast': 'Figura',
    'be': 'Малюнак',
    'br': 'Sifr',
    'bg': 'Фигура',
    'ca': 'Figura',
    'ca-ES-Valencia': 'Figura',
    'zh-CN': '数字',
    'da': 'Figur',
    'nl': 'Figuur',
    'en-AU': 'Figure',
    'en-CA': 'Figure',
    'en-NZ': 'Figure',
    'en-ZA': 'Figure',
    'en-GB': 'Figure',
    'en-US': 'Figure',
    'eo': 'Figuro',
    'fr': 'Figure',
    'gl': 'Figura',
    'de-DE': 'Abbildung',
    'de-AU': 'Abbildung',
    'de-CH': 'Abbildung',
    'el': 'Φιγούρα',
    'he': 'דמות',
    'is': 'Mynd',
    'it': 'Figura',
    'ja': '図',
    'km': 'រូបភាព',
    'lt': 'Pav',
    'ml': 'ചിത്രം',
    'nb-NO': 'Figur',
    'nn-NO': 'Figur',
    'fa': 'شکل',
    'pl': 'Postać',
    'pt-BR': 'Figura',
    'pt-PT': 'Figura',
    'ro': 'Figura',
    'ru': 'фигура',
    'tr': 'şekil',
    'sr-SP-Cy': 'Фигура',
    'sr-SP-Lt': 'Figura',
    'sk': 'Figúra',
    'sl': 'Slika',
    'es': 'Figura',
    'sv': 'Figur',
    'ta': 'படம்',
    'tl': 'Figure',
    'uk': 'Малюнок'
}

const EQUATION = {
    'af-ZA': 'Vergelyking',
    'sq-AL': 'Ekuacioni',
    'ar': 'معادلة',
    'ast': 'Ecuación',
    'be': 'Yраўненне',
    'br': 'Kevatalenn',
    'bg': 'Yравнение',
    'ca': 'Equació',
    'ca-ES-Valencia': 'Equació',
    'zh-CN': '方程',
    'da': 'Ligning',
    'nl': 'Vergelijking',
    'en-AU': 'Equation',
    'en-CA': 'Equation',
    'en-NZ': 'Equation',
    'en-ZA': 'Equation',
    'en-GB': 'Equation',
    'en-US': 'Equation',
    'eo': 'Ekvacio',
    'fr': 'Équation',
    'gl': 'Ecuación',
    'de-DE': 'Gleichung',
    'de-AU': 'Gleichung',
    'de-CH': 'Gleichung',
    'el': 'εξίσωση ',
    'he': 'משוואה',
    'is': 'Jöfnu',
    'it': 'Equazione',
    'ja': '方程式',
    'km': 'សមីការ',
    'lt': 'Lygtis',
    'ml': 'സമവാക്യം',
    'nb-NO': 'Ligning',
    'nn-NO': 'Likning',
    'fa': 'معادله',
    'pl': 'Równanie',
    'pt-BR': 'Equação',
    'pt-PT': 'Equação',
    'ro': 'Ecuaţie',
    'ru': 'Yравнение',
    'tr': 'Denklem',
    'sr-SP-Cy': 'Једначина',
    'sr-SP-Lt': 'Jednačina',
    'sk': 'Rovnica',
    'sl': 'Enačba',
    'es': 'Ecuación',
    'sv': 'Ekvation',
    'ta': 'சமன்பாடு',
    'tl': 'Equation',
    'uk': 'Pівняння'
}


const TABLE = {
    'af-ZA': 'Tabel',
    'sq-AL': 'tabelë',
    'ar': 'الطاولة',
    'ast': 'Tabla',
    'be': 'Табліца',
    'br': 'Taolenn',
    'bg': 'маса',
    'ca': 'Taula',
    'ca-ES-Valencia': 'Taula',
    'zh-CN': '表',
    'da': 'Tabel',
    'nl': 'Tafel',
    'en-AU': 'Table',
    'en-CA': 'Table',
    'en-NZ': 'Table',
    'en-ZA': 'Table',
    'en-GB': 'Table',
    'en-US': 'Table',
    'eo': 'Tabelo',
    'fr': 'Table',
    'gl': 'Táboa',
    'de-DE': 'Tabelle',
    'de-AU': 'Tabelle',
    'de-CH': 'Tabelle',
    'el': 'Τραπέζι',
    'es': 'Tabla',
    'he': 'שולחן',
    'is': 'Tafla',
    'it': 'Tavolo',
    'ja': '表',
    'km': 'តារាង',
    'lt': 'Lentelė',
    'ml': 'മേശ',
    'nb-NO': 'Tabel',
    'nn-NO': 'Tabel',
    'fa': 'جدول',
    'pl': 'Stół',
    'pt-BR': 'Tabela',
    'pt-PT': 'Tabela',
    'ro': 'Tabel',
    'ru': 'Таблица',
    'tr': 'Tablo',
    'sr-SP-Cy': 'Табела',
    'sr-SP-Lt': 'Tabela',
    'sk': 'Stôl',
    'sl': 'Tabela',
    'sv': 'Tabell',
    'ta': 'மேசை',
    'tl': 'Table',
    'uk': 'Таблиця'
}

const PHOTO = {
    'af-ZA': 'Foto',
    'sq-AL': 'Foto',
    'ar': 'صورة فوتوغرافية',
    'ast': 'Foto',
    'be': 'Фота',
    'br': 'Foto',
    'bg': 'снимка',
    'ca': 'Foto',
    'ca-ES-Valencia': 'Foto',
    'zh-CN': '照片',
    'da': 'Foto',
    'nl': 'Foto',
    'en-AU': 'Photo',
    'en-CA': 'Photo',
    'en-NZ': 'Photo',
    'en-ZA': 'Photo',
    'en-GB': 'Photo',
    'en-US': 'Photo',
    'eo': 'Foto',
    'fr': 'Photo',
    'gl': 'Foto',
    'de-DE': 'Foto',
    'de-AU': 'Foto',
    'de-CH': 'Foto',
    'el': 'φωτογραφία',
    'he': 'תמונה',
    'is': 'Mynd',
    'it': 'Foto',
    'ja': '写真',
    'km': 'រូបថត',
    'lt': 'Nuotrauka',
    'ml': 'ഫോട്ടോ',
    'nb-NO': 'Bilde',
    'nn-NO': 'Bilde',
    'fa': 'عکس',
    'pl': 'Zdjęcie',
    'pt-BR': 'Foto',
    'pt-PT': 'Foto',
    'ro': 'Fotografie',
    'ru': 'Фото',
    'tr': 'Fotoğraf',
    'sr-SP-Cy': 'Фото',
    'sr-SP-Lt': 'Foto',
    'sk': 'Fotografie',
    'sl': 'Fotografija',
    'es': 'Foto',
    'sv': 'Foto',
    'ta': 'புகைப்பட',
    'tl': 'Larawan',
    'uk': 'Фото'
}

export const CATS = {
    'figure': FIGURE,
    'equation': EQUATION,
    'table': TABLE,
    'photo': PHOTO,
}
